//
//  AGCaseInsensitiveStringDictionary.h
//  AraeliumFoundation
//
//  Created by Seth Willits on 2/6/14.
//  Copyright (c) 2014 Araelium Group. All rights reserved.
//

#import <Foundation/Foundation.h>


//! This custom dictionary uses strings as keys, with a case insensitive lookup. The keys
//! @"foo" and @"Foo" will both return the same value set using the key @"FOO".
//! The performance of the lookup in this class is 6 times faster than using a normal dictionary
//! with key.lowercaseString. Insert of a non-existent key and removal are both about 3x slower.
//! The best use of this dictionary is one in mutation is infrequent, and lookups occur in great
//! numbers.
//! Is also thread-safe.
@interface AGMutableCaseInsensitiveStringDictionary : NSObject
{
	NSMutableDictionary * _dict;
	NSMutableDictionary * _lowerToVariants;
	NSMutableDictionary * _variantToLower;
}

- (void)setObject:(id)value forKey:(NSString *)key;
- (id)objectForKey:(NSString *)key;
- (void)removeObjectForKey:(NSString *)key;
- (void)removeAllObjects;

- (NSArray *)allValues;

//! Will return just the lowercase variants of all keys
- (NSArray *)allKeys;

@end
